#!perl

#
# Módulo de prueba contra regresiones en salida
# Antonio García Domínguez
# $Id$
#
use strict;
use warnings;

use Test::More;
use XML::Validate;
use XML::SemanticDiff;
use XML::Parser;

use Cwd;
use File::Copy;
use File::Spec;

use ACL2::Demostracion;
use ACL2::Test::ManejDetalleDiff;

# Nombres de los documentos a usar para pruebas
my @docsPrueba = map { m!.*/(.*?)\.lisp!i } <t/testInputs/*.lisp>;

# Avisamos a Test::More del número de pruebas a realizar
plan tests => 2 * scalar @docsPrueba;

# Generamos el validador de XML
my $validador = new XML::Validate(Type => 'BestAvailable');

foreach my $i (@docsPrueba) {
    my $fuenteLisp  = "t/testInputs/${i}.lisp";
    my $fuenteDem   = "t/testInputs/${i}.lisp.out";
    my $salidaXML   = "t/testInputs/${i}.lisp.xml";
    my $esperadoXML = "t/expectedOutputs/${i}.xml.test";

    my $demo = new ACL2::Demostracion("ruta_lisp" => $fuenteLisp);
    $demo->actualizar;

    my $cadenaXML = volcarFichero($salidaXML);

    ##################################################
    # Validamos la salida XML contra la DTD incrustada
    ##################################################

    my $valido = $validador->validate($cadenaXML);
    ok($valido, "El fichero ${salidaXML} es válido y sigue la DTD");
    if (!$valido) {
	print "ERRORES DE VALIDACION:\n" .
	    "------------------------\n";

	while (my ($clave, $contenido) = each %{$validador->last_error()}) 
	{
	    print "${clave}: $contenido\n";
	}
	
	print "------------------------\n";
    }

    ###################################################
    # Comprobamos que la salida es la esperada
    ###################################################

    # Si no existe la salida esperada, asumimos que es la primera vez
    # que se ejecuta el test y copiamos la obtenida
    if (! -e $esperadoXML) {
	copy($salidaXML, $esperadoXML) 
	    or die "No se pudo copiar $salidaXML a $esperadoXML";
    }

    my $cadEsperadoXML  = volcarFichero($esperadoXML);
    my $hay_cambios_imp = compararSalidas($cadenaXML, $cadEsperadoXML);
    ok($hay_cambios_imp == 0,  
       "El fichero ${salidaXML} coincide con la versión esperada");

}

=head2 volcarFichero($ruta)

Uso: Devuelve una cadena con el contenido del fichero con la ruta relativa $ruta.

=cut

sub volcarFichero {
    my ($ruta) = @_;

    open(F_VOLCARFICHERO, $ruta) or die "No se pudo abrir $ruta";
    my $res = join '', <F_VOLCARFICHERO>;
    close(F_VOLCARFICHERO);

    return $res;
}

=head2 compararSalidas($doc1, $doc2)

Uso: Devuelve 1 si existen diferencias importantes entre las cadenas
con las salidas XML $doc1 y $doc2, o 0 de lo contrario.

=cut

sub compararSalidas {
    my ($cadenaXML, $cadEsperadoXML) = @_;
    my $diff = new XML::SemanticDiff(
	keepdata => 1,
	diffhandler  => new ACL2::Test::ManejDetalleDiff( keepdata => 1 )
    );
    my $hay_cambios_importantes = 0;
    
    foreach my $change ($diff->compare($cadenaXML, $cadEsperadoXML) )
    {
	my ($event, $elem, $attr, $context) = ($change->{event}, 
					       $change->{element_name}, 
					       $change->{attribute_name},
					       $change->{context});
	
	# Filtramos los cambios no importantes
	unless ($event eq 'attribute_value' 
		&& ($elem eq 'time' 
                    || $elem eq 'script' 
                    || $elem eq 'acl2_version' 
                    || $attr eq 'name' && $context =~ m!certifybook!i
                    || normalizeWhitespace($change->{old_value}) eq normalizeWhitespace($change->{new_value}))
		|| $context =~ m!certifybook.*/paragraph\[1\]!i
		|| $context =~ m!certifybook.*/compilation.*/paragraph\[2\]!i
		|| $context =~ m!certifybook.*/write_cert.*/sexpr\[1\]!i
		|| $context =~ m!includebook.*/paragraph\[[0-9]+\]!i)
	{
	    print "$change->{message}: contexto $change->{context}\n";

	    if ($event eq 'attribute_value' || $event eq 'element_value') {
		print "Se esperaba:\n" 
		    . $change->{old_value}
		    . "\nSe obtuvo:\n" . $change->{new_value} . "\n";
	    }
	    
	    $hay_cambios_importantes = 1;
	}
    }

    return $hay_cambios_importantes;
}

sub normalizeWhitespace {
    my ($a) = @_;

    $a =~ s/^\s+//g;
    $a =~ s/\s+$//g;
    $a =~ s/\s{2,}/ /g;

    return $a;
}
